# R-script:
# Use of Min/Max Autocorrelation Factors (MAF) for combining raw indices 
# into time continuous multivariate indicators 
#       
# EU program Fisboat, DG-Fish, STREP n 502572
# Authors : M.Woillez, J.Rivoirard (Mines-ParisTech)
# Last update : 15/12/2008


# source R functions
source("MAF_functions.r")

# Read the data

# The table to be read must contain all the indices which would be needed for 
# describing a fish stock component (e.g. the spatial pattern): 
# Spatial indices at life stage (or at age) 

# Abundance indices at life stage (or at age) are also needed 
# to identify potential relationships between spatial and population dynamic

# Refers to the table given as example for the input structure 
# Table must contained "Year" and "Age" as variables.
dat<-read.table("codNS_85-04_Groups&Spa.txt",header=T)
name<-"codNS"

# Create variables:
# z the values of indices. 
# t the corresponding time coordinates.
ni <- dim(dat)[1]; nj <- dim(dat)[2]
z <- dat[,4:nj]

# Ln-tranform abundance
jln <- match("Abundance",dimnames(z)[[2]])
z[jln] <- log(z[jln])

# Check the presence of "Age" variable
Age<-as.character(unique(z$Age))
if(sum(is.na(Age))) 
    stop(paste("Age is not properly defined"))
    
# Select indices:
Ind <- names(z[-(1:3)])
jInd <- length(Ind)

# Loop on Ages
res.maf<-vector("list",length(Age))
res.reg<-vector("list",2)
for(i in 1:length(Age)){

    # Select data on "Age" and set MAF formula 
    t <- na.omit(z)$Year[na.omit(z)$Age==Age[i]]
    zInd<-z[z$Age==Age[i],sort(match(c("Year","Age","Abundance",Ind),names(z)))]
    zInd<-na.omit(data.frame(zInd,row.names=t))
    Ind.n<-sort(match(Ind,names(zInd)))
    fo<-as.formula(paste("~",sapply(list(names(zInd[Ind.n])), paste, collapse="+"),sep=""))

    # Test on indices
    # function 'prcomp' fails if its variance is null (i.e. values are equal)
    # In that case, analysis is done without this index
    for(j in 1:length(zInd)){ 
        if(is.numeric(zInd[,j])){
            va<-var(zInd[,j])
            if(!is.na(va)&va==0){
                zInd<-zInd[names(zInd)!=names(zInd[j])]
                Ind.n<-sort(match(Ind,names(zInd)))
                fo<-as.formula(paste("~",sapply(list(names(zInd[Ind.n])), 
                    paste, collapse="+"),sep=""))
            }
        }
    }

    # define outputs to store MAFs realisations 
    # where a normal random noise is added to indices   
    res.noise <- vector("list",jInd)
    res <- vector("list",2)
    nreal <- 1000
    for(j in 1:jInd)
        res.noise[[j]] <- res

    # set the random seed
    seed <- 310882
    set.seed(seed)

    # loop: nreal realizations is performed               
    for(j in 1:nreal){
            
        # work on centred and normed indices
        # add of a normal random noise with mean 0 and 
        # variance 0.1*(nb.indices/(nb.years-1))
        scaled.zInd <- scale(zInd[Ind.n])
        nij <- dim(scaled.zInd)
        noise <- matrix(rnorm(prod(nij),0,sqrt(0.1*(nij[2]/(nij[1]-1)))),nij[1],nij[2])
        new.zInd <- as.data.frame(scaled.zInd + noise)
            
        # Compute MAFs
        resmaf<-maf(fo,new.zInd)
                
        # MAFs are ordered according to a reference
        # the reference is the first realisation
        lng <- length(resmaf$x[1,])
        if(j==1)
            test <- resmaf
        for(k in 1:lng){                
            # the reference is ordered so as to have 50% of the MAFs values as positive
            if(sum(test$x[,k]>=0)/length(test$x[,k])<0.5){
               test$x[,k] <- test$x[,k] * -1
               test$rotation[,k] <- test$rotation[,k] * -1
            }         
            # the following realisations will be order so as to have 50% of the MAFs values 
            # with the same sign than the reference 
            if(sum((test$x[,k]>=0)==(resmaf$x[,k]>=0),na.rm=T)/length(test$x[,k])<0.5){
                resmaf$x[,k] <- resmaf$x[,k] * -1
                resmaf$rotation[,k] <- resmaf$rotation[,k] * -1
            }
        }
                        
        # save results in outputs
        t.l <- seq(t[1],t[length(t)],1)                       
        for(k in 1:lng){
            res.noise[[k]][[1]] <- rbind(res.noise[[k]][[1]],as.numeric(resmaf$x[,k][match(t.l,t)]))
            res.noise[[k]][[2]] <- rbind(res.noise[[k]][[2]],as.numeric(resmaf$rotation[,k]))
        }
    }

    # Number of MAFs retained
    nbmafs <- 3
        
    # Compute and save
    # - median loadings of the nreal previous realisations   
    # - median MAF = median loadings * centred and normed indices 
    for(j in 1:lng){
        med.rot <- apply(res.noise[[j]][[2]],2,median)
        resmaf$rotation[,j] <- med.rot
        rot <- matrix(rep(med.rot,nij[1]),nij[1],byrow=T)
        x <- apply(rot * scaled.zInd,1,sum)
        resmaf$rotation[,j] <- resmaf$rotation[,j]/sd(x)        
        resmaf$x[,j] <- x/sd(x)
    }

    # Loop on the retained MAFs
    # Plots produced for each MAF: 
    # - its time series
    # - its variogram
    # - the loadings of each variable in the MAF (the highest is highlighted)

    maf.vg <- vector("list",nbmafs)

    for(j in 1:nbmafs){
        x11(width=9,heigh=3)
        par(mfrow = c(1,3))
        par("mar"=c(2.5,2.5,2.5,1))
        maf.x<-as.numeric(resmaf$x[,j][match(t.l,t)])
        barplot(aperm(matrix(maf.x)),names.arg=matrix(t.l),beside=F,space=0,col=8,
            cex.names=.7,main=paste("MAF ",j," (",name," ",Age[i],")",sep=""),
            ylim=c(-3.5,3.5))

        maf.vg[[j]]<-vg1d.reg(maf.x)
        plot(maf.vg[[j]],type="o",main="",ylim=c(0,3))
        abline(h=1,lty=2)
        title("Variogram")

        maf.rot<-resmaf$rotation[,j]
        plot(0,0,ylim=c(-max(ceiling(abs(maf.rot))),max(ceiling(abs(maf.rot)))),
            xlim=c(0,jInd),type="n",xlab="",ylab="",xaxt="n",main="Loadings")
        segments(x0=seq(0.5,jInd-.5,1),y0=rep(0,jInd),x1=seq(0.5,jInd-.5,1),y1=maf.rot)
        abline(h=0)
        Col<-rep(1,jInd)
        Col[seq(1,jInd,1)[abs(maf.rot)==max(abs(maf.rot))]]<-3
        text(seq(0.5,jInd-.5,1),maf.rot,names(zInd[Ind.n]),cex=.75,col=Col)

        dev.print(png, file=paste("MAF.",name,".",Age[i],".fig",j,".png", sep=""), width=1200, height=400)
        dev.copy2eps(file=paste("MAF.",name,".",Age[i],".fig",j,".eps",sep=""))
    }

    # Time series of the MAF 
    # Time series of the log of the abundance
    # Scatterplot and regression between the log of the abundance and the MAF 
    # Delayed correlations between the MAF and the log of the abundance
    for(j in 1:nbmafs){       
        x11(width=6,heigh=6)
        par(mfrow = c(2,2))
        par("cex.main"=1)
        par("mar"=c(4.5,4.5,2,1))
        maf.x<-as.numeric(resmaf$x[,j][match(t.l,t)])
        barplot(aperm(matrix(maf.x)),names.arg=matrix(t.l),beside=F,space=0,col=8,
            main=paste("median MAF ",j," (",name," ",Age[i],")",sep=""),
            ylim=c(-3.5,3.5))

        logAb<-as.numeric(zInd$Abundance[match(t.l,t)])
        plot(t.l,logAb,main=paste("log Abundance (",name," ",Age[i],")",sep=""),
            type="l",xlab="",ylab="")
        reg<-ploc(maf.x,logAb,xlab=paste("MAF ",j,sep=""),
            ylab=paste("log Abundance ",name," ",Age[i],sep=""))
        res.reg[[1]]<-c(res.reg[[1]],round(summary(reg)$r.squared,digit=3))
        res.reg[[2]]<-c(res.reg[[2]],round(summary.aov(reg)[[1]]$"Pr(>F)"[1],3))

        crosscor(maf.x,logAb,lag.max=10)
        title(main=paste("MAF ",j," vs log Abundance at lag",sep=""))
                  
        dev.print(png, file=paste("MAF.",name,".",Age[i],".fig",nbmafs+j,".png",sep=""), width=800, height=800)
        dev.copy2eps(file=paste("MAF.",name,".",Age[i],".fig",nbmafs+j,".eps",sep=""))
    } 
    res.maf[[i]]<-resmaf

    # Shut down all open graphic devices
    # when the analysis of one age is finished  
    #graphics.off()
}

names(res.maf)<-Age
for(i in 1:2){
    res.reg[[i]] <- matrix(res.reg[[i]],nrow=nbmafs,byrow=F)
    row.names(res.reg[[i]]) <- paste("MAF",1:nbmafs,sep="")
    res.reg[[i]] <- data.frame(res.reg[[i]])
    names(res.reg[[i]]) <- as.character(unique(z$Age))
}
names(res.reg)<-c("R.squared","P.value")

# Clean workspace
rm(Age,Col,fo,i,Ind,Ind.n,j,jInd,jln,k,lng,logAb,
maf.rot,maf.vg,maf.x,med.rot,name,nbmafs,new.zInd,ni,nij,nj,noise,
nreal,reg,res,res.noise,resmaf,rot,scaled.zInd,seed,t,t.l,test,va,x,z,zInd)

# Results
res.maf
res.reg
