"maf" <-
function(formula,data)
{
#===============================================================================
# Min/Max Autocorrelation Factors (MAF) in time (or 1D)
#
# EU program Fisboat, DG-Fish, STREP n 502572
# Authors : M.Woillez, J. Rivoirard, Mines-ParisTech
# Last update : 01 march 2008
#
# Description:
# The MAF are a multivariate statistical method, that allows the set of
# initial variables to be decomposed into factors. The autocorrelation of
# which decreases, or the variogram increases, when going from the first
# factors to the last ones. Hence the very first factors extract the part of
# the variables which is the most continuous in time.
#
# Details:
# The present routine makes twice use of PCA. Here 'prcomp' is used for PCA
# instead of 'princomp', as it allows less samples (e.g. years) than variables.
# After original variables are turned into normalized PC, MAF are obtained as
# the PC of their increments; their order is then reversed to begin with the
# smallest variograms (i.e. the half variances of increments).
#
# Note:
# The number of MAF cannot exceed the number of variables, nor the number of
# year increments (number of sampled years - 1). If the number of variables
# tends to be larger than the number of sampled years, the MAF no.i (i=1, ...)
# tends to have period (nb years - 1)*2/i. In particular there will be
# evidence of a high continuity with period (nb years -1)*2 for MAF1,
# (nb years -1) for MAF2, etc, whatever the data, which may not be significant
# out of this series e.g. for additional years. The number of variables should
# then be better reduced well below the number of sampled years.
#
# Inputs:        
#   formula   A formula with no response variable, referring only 
#             to numeric variables.
#
#   data      A data frame containing the variables in the formula.
#
#===============================================================================

    if(length(data[,1])==1) stop("Cannot be run with only one sample.")

# PCA of original variables
    bidpr<-prcomp(formula,data,scale=TRUE)
    bidn<-sum(bidpr$sdev>1e-10)
    if(bidn<2) stop("stop: strictly less than two PCs")
    bidpr$sdev<-bidpr$sdev[1:bidn]
    bidpr$rotation<-bidpr$rotation[,1:bidn]
    bidpr$x<-bidpr$x[,1:bidn]

# normalization of PCs
    bidim<-dim(bidpr$x)
    bidprsdev<-matrix(rep(bidpr$sdev,each=bidim[1]),ncol=bidim[2])
    bidprx<-bidpr$x/bidprsdev
    bidim<-dim(bidpr$rotation)
    bidprsdev<-matrix(rep(bidpr$sdev,each=bidim[1]),ncol=bidim[2])
    bidpr$rotation<-bidpr$rotation/bidprsdev

# PCA of increments of normalized PCs
    biddtab<- diff(bidprx)
    biddpr<-prcomp(biddtab,center=F,scale=F)
    biddnmaf<-sum(biddpr$sdev>1e-10)
    if(biddnmaf<2) stop("stop: strictly less than two MAFs")
    biddpr$sdev<-biddpr$sdev[1:biddnmaf]
    biddpr$rotation<-biddpr$rotation[,1:biddnmaf]
    biddpr$x<-biddpr$x[,1:biddnmaf]
    biddx<-biddpr$x

# store complete transformation as a PCA output from original data
    maf<-bidpr
    maf$sdev <- biddpr$sdev
    biddim<-dim(biddpr$x)
    maf$rotation<-bidpr$rotation%*%biddpr$rotation
    maf$x<-bidprx%*%biddpr$rotation

# reverse the order of MAFs
# in order to begin with lowest eigenvalues i.e. lowest variograms

    maf$sdev<-.5*(maf$sdev^2)*(biddim[1]-1)/biddim[1]
    names(maf$sdev)<-rev(names(maf$sdev))
    maf$sdev<-rev(maf$sdev)
    lng<-length(dimnames(maf$rotation)[[2]])
    maf$rotation <- maf$rotation[,seq(lng,1)]
    dimnames(maf$rotation)[[2]]<-paste("MAF",1:lng,sep="")
    lng<-length(dimnames(maf$x)[[2]])
    maf$x<-maf$x[,seq(lng,1)]
    dimnames(maf$x)[[2]]<-paste("MAF",1:lng,sep="")
    maf
}

"vg1d.reg" <-
function(x)
{
#===============================================================================
# Regular 1d variogram
#
# EU program Fisboat, DG-Fish, STREP n 502572
# Authors : M.Woillez, J. Rivoirard (Mines-ParisTech)
# Last update : 01 march 2008
#
#===============================================================================

lng<-length(x)
res<-numeric(0)
    for(h in 1:(lng-1))
    res<-c(res,0.5*mean(diff(x,lag=h)^2,na.rm=T))
res<-cbind(1:(lng-1),res)
res
}

"vg1d.irreg" <-
function(t,x)
{
#===============================================================================
# Irregular 1d variogram
#
# EU program Fisboat, DG-Fish, STREP n 502572
# Authors : P.Petitgas (Ifremer), M.Woillez, J. Rivoirard (Mines-ParisTech)
# Last update : 01 march 2008
#
#===============================================================================

d<-abs(outer(t,t,"-"))
h<-sort(unique(d[lower.tri(d)]))
res<-numeric(0)
for (k in 1:length(h)) {
    d2x<-outer(x,x,"-")^2
    res<-c(res,0.5*mean(d2x[d==h[k]],na.rm=T)) }
res<-cbind(h,res)
res
}

"ploc" <-
function(x,y=NA,plot=T,add=F,...)
{
#===============================================================================
# Bivariate plot and linear regression
#
# EU program Fisboat, DG-Fish, STREP n 502572
# Authors : M.Woillez, J. Rivoirard (Mines-ParisTech)
# Last update : 01 march 2008
#
#===============================================================================

    if(length(y) == 1 && is.na(y)){
        y<-x
        x<-seq(1,length(y),1)
    }
    lab<-paste("R= Null   F value= Null   Pr(>F)=Null",sep="")
    if(sum(x*y,na.rm=T)!=0){
        reg<-lm(y~x)
        bid<-summary(reg)
        bidaov<-summary.aov(reg)
        if(bid$r.squared!=0 & !is.na(bid$r.squared))
            lab<-paste("R=",round(bid$r.squared,digit=3),"   F value=",
                round(bidaov[[1]]$"F value"[1],3),"   Pr(>F)=",
                round(bidaov[[1]]$"Pr(>F)"[1],3),sep="")
        }
    if(plot==T){
    par("cex.main"=1)
    par("mar"=c(4.5,4.5,2,1))
    if(add==T)
        points(x,y,main=lab,...)
    else
        plot(x,y,main=lab,...)
    if(sum(x*y,na.rm=T)!=0 & bid$r.squared!=0 & !is.na(bid$r.squared))
        abline(bid$coefficients[,1],col=2)
    }
    reg
}

"crosscor" <-
function (x, y, type="1", lag.max=NA)
{
#===============================================================================
# Cross correlation between two variables x and y
#
# EU program Fisboat, DG-Fish, STREP n 502572
# Authors : M.Woillez, J. Rivoirard (Mines-ParisTech)
# Last update : 01 march 2008
#
#===============================================================================

    if(length(x)!=length(y))
        stop("x and y must have the same length")
    res<-numeric(0)
    bidn<-length(x)-1
    if(bidn > lag.max & !is.na(lag.max))
        bidn<-lag.max
    for(i in -bidn:bidn){
        if(i<0){
            xx<-c(x,rep(NA,abs(i)))
            yy<-c(rep(NA,abs(i)),y)
        }
        if(i>=0){
            xx<-c(rep(NA,abs(i)),x)
            yy<-c(y,rep(NA,abs(i)))
        }
        bid<-na.omit(data.frame(xx,yy))
        xx<-bid[,1]
        yy<-bid[,2]
        res<-c(res,(mean((xx-mean(xx,na.rm=T))*(yy-mean(yy,na.rm=T)),na.rm=T))/
            (sqrt((sum(xx^2)/length(xx)) - ((sum(xx)^2)/length(xx)^2))
            *sqrt((sum(yy^2)/length(yy)) - ((sum(yy)^2)/length(yy)^2))))
    }
    plot(seq(-bidn,bidn,1),res,type="h",xlab="Lag",ylab="Correlation",ylim=c(-1,1))
    abline(h=c(-1,0,1),lty=c(2,1,2))
    res
}

"nugget" <-
function(h)
{
#===============================================================================
# Variogram form of the nugget model
#
# Routine from Geostatistics for Estimating Fish Abundance (GEFA)
# & EU program Fisboat, DG-Fish, STREP n 502572
# Authors : M.Woillez (Mines-ParisTech), N.Bez (IRD) 
#           and J.Rivoirard (Mines-ParisTech)
# Last update : 01 march 2008
#
#===============================================================================

	null <- (h == 0)
	g <- h
	g[null] <- 0
	g[!null] <- 1
	g
}

"spheric" <-
function(h)
{
#===============================================================================
# Variogram form of the spherical model
#	with sill 1 and range 1
#
# Routine from Geostatistics for Estimating Fish Abundance (GEFA)
# & EU program Fisboat, DG-Fish, STREP n 502572
# Authors : M.Woillez (Mines-ParisTech), N.Bez (IRD) 
#           and J.Rivoirard (Mines-ParisTech)
# Last update : 01 march 2008
#
#===============================================================================

	h <- abs(h)
	superieur <- (h > 1)
	g <- h
	g[superieur] <- 1
	inferieur <- (!superieur) & (!is.na(h))
	g[inferieur] <- 0.5 * h[inferieur] * (3 - h[inferieur]^2)
	g
}

"cubic" <-
function(h)
{
#===============================================================================
# Variogram form of the cubic model
#	with sill 1 and range 1
#
# Routine from Geostatistics for Estimating Fish Abundance (GEFA)
# & EU program Fisboat, DG-Fish, STREP n 502572
# Authors : M.Woillez (Mines-ParisTech), N.Bez (IRD) 
#           and J.Rivoirard (Mines-ParisTech)
# Last update : 01 march 2008
#
#===============================================================================

	h <- abs(h)
	superieur <- (h > 1)
	g <- h
	g[superieur] <- 1
	inferieur <- (!superieur) & (!is.na(h))
	g[inferieur] <- h[inferieur]^2 * (7 - (35/4) * h[inferieur] + 3.5 * h[
		inferieur]^3 - 0.75 * h[inferieur]^5)
	g
}

"powervg" <-
function(h, alpha)
{
#===============================================================================
# Variogram form of the power model
#
# Routine from Geostatistics for Estimating Fish Abundance (GEFA)
# & EU program Fisboat, DG-Fish, STREP n 502572
# Authors : M.Woillez (Mines-ParisTech), N.Bez (IRD) 
#           and J.Rivoirard (Mines-ParisTech)
# Last update : 01 march 2008
#
#===============================================================================

        h <- abs(h)
        g <- h
        sel <- !is.na(h)
        g[sel] <- h[sel]^alpha
        g
}

"okun1d" <- function(z,x,xk,model)
{
#===============================================================================
# One dimensional ordinary kriging with unique neighborhood 
#
# Routine from Geostatistics for Estimating Fish Abundance (GEFA)
# & EU program Fisboat, DG-Fish, STREP n 502572
# Authors : M.Woillez (Mines-ParisTech), N.Bez (IRD) 
#           and J.Rivoirard (Mines-ParisTech)
# Last update : 01 march 2008
#
# Inputs:
#   z         Data values. NA values are NOT ALLOWED.
#   x         A vector of x-coordinates of data points.  
#   xk        A vector of x-coordinates of target points. 
#   model     An appropriate structural model (unevaluated expression).
#
# Outputs:
#   zkrig     A vector of the kriged values at target points.
#   vark      A vector of the kriging variance at target points.
#
#===============================================================================

    nx <- length(x)
    nxk <- length(xk)

# Built and invert left member of kriging system
    hx <- outer(x, rep(1, nx)) - outer(rep(1, nx), x)
    Kij <- eval({h<-hx; model})
    Kij <- cbind(Kij, rep(1, nx), deparse.level = 0)
    Kij <- rbind(Kij, c(rep(1, nx), rep(0, 1)), deparse.level = 0)

# Built right member of kriging system
    hx <- outer(x, rep(1, nxk)) - outer(rep(1, nx), xk)
    Kix <- eval({h<-hx; model})
    Kix <- rbind(Kix, rep(1, nxk), deparse.level = 0)

# Process the kriging system 
    solution <- solve(Kij)%*%Kix
    lambda <- solution[1:nx,]

# Evaluation on target points
    zkrig <- t(lambda) %*% z
    vark <- as.vector(t(solution * Kix) %*% rep(1,length(solution[,1])))
    res <- data.frame(zkrig,vark)
    res
}
