# R-script:
# Use of Min/Max Autocorrelation Factors (MAF) for selecting
# informative indicators 
#       
# EU program Fisboat, DG-Fish, STREP n 502572
# Authors : M.Woillez, J.Rivoirard (Mines-ParisTech) and P.Petitgas (Ifremer)
# Last update : 15/12/2008


# source R functions
source("MAF_functions.r")

# Read the data
# The table to be read must contain all the indices which would be needed for 
# the diagnostic of the stock: 
# Biological indices and Spatial indices at life stage (or at age) 
# Refers to the table given as example for the input structure 

# North Sea cod stock
dat<-read.table("codNS_85-04_Groups&Bio&Spa.txt",header=T)
name <- "codNS"

# Create variables:
# z the values of indices. t the corresponding time coordinates.
ni <- dim(dat)[1]; nj <- dim(dat)[2]
z <- dat[,5:nj]
t <- dat[,4]

# exclude some unwanted indices
z <- z[-c(31,32)]

# Ln-tranform abundance
jln <- match(paste("Abundance.",c("Ma","Im","Re"),sep=""),dimnames(z)[[2]])
z[jln] <- log(z[jln])

# One-lag variogram computation
# will take in charge lags <> 1. 
# g1 = average g on the first tmax years
nz <- length(z)
g1 <- numeric(0)
tmax <- 1
for (i in 1:nz){ 
    g <- vg1d.irreg(t,scale(z[i]))
    g1<-c(g1,mean(g[g[,1]<=tmax,2]))
}
names(g1) <- dimnames(z)[[2]]
g1 <- sort(g1)
Ind <- names(g1)

# Plot results
x11(5,8)
par("mar"=c(5,2,5,2))
plot(g1,rev(1:nz),pch="+",ylab="Indices",xlab="Variogram at lag 1",main=name,
    yaxt="n",xlim=c(0,max(g1)+.5))
text(g1,rev(1:nz),label=Ind,pos=4,cex=.5)
abline(v=1,lty=2)
res.ts.vario.at.lag.1 <- data.frame(Indices=names(g1),Vario.at.lag.1=as.numeric(g1))

# Save figures
dev.print(png, file=paste("MAF.",name,".fig0.png",sep=""), width=500, height=800)
dev.copy2eps(file=paste("MAF.",name,".fig0.eps",sep=""))

# Total number of indices
nz

# Number of indices that are easy to follow
# i.e. indices with value at lag 1 of the normed variogram below 1 
length(g1[g1<1])

# Selection of the indices to be included in the MAF
# Either select as many clumps as possible with a total number of indices 
# below the number of studied years, or retain all indices with variogram 
# at lag 1 below 1
jInd <- 12 
#jInd <- length(g1[g1<1])
Ind <- Ind[1:jInd]

# Set MAF formula 
zInd<-z[,sort(match(Ind,names(z)))]
zInd<-na.omit(data.frame(zInd,row.names=t))
fo<-as.formula(paste("~",sapply(list(names(zInd)), paste, collapse="+"),sep=""))
 
# Test on indices
# function 'prcomp' fails if its variance is null (i.e. values are equal)
# In that case, analysis is done without this index
for(i in 1:jInd){ 
    if(is.numeric(zInd[,i])){
        va<-var(zInd[,i])
        if(!is.na(va)&va==0){
            zInd<-zInd[names(zInd)!=names(zInd[i])]
            fo<-as.formula(paste("~",sapply(list(names(zInd)), 
                paste, collapse="+"),sep=""))
        }
    }
}
   
# define outputs to store MAFs realisations 
# where a normal random noise is added to indices   
res.noise <- vector("list",jInd)
res <- vector("list",2)
nreal <- 1000
for(i in 1:jInd)
    res.noise[[i]] <- res

# set the random seed
seed <- 310882
set.seed(seed)

# loop: nreal realizations is performed               
for(i in 1:nreal){
        
    # work on centred and normed indices
    # add of a normal random noise with mean 0 and 
    # variance 0.1*(nb.indices/(nb.years-1))
    scaled.zInd <- scale(zInd)
    nij <- dim(scaled.zInd)
    noise <- matrix(rnorm(prod(nij),0,sqrt(0.1*(nij[2]/(nij[1]-1)))),nij[1],nij[2])
    new.zInd <- as.data.frame(scaled.zInd + noise)
        
    # Compute MAFs
    resmaf<-maf(fo,new.zInd)
            
    # MAFs are ordered according to a reference
    # the reference is the first realisation
    lng <- length(resmaf$x[1,])
    if(i==1)
        test <- resmaf
    for(j in 1:lng){
            
        # the reference is ordered so as to have 50% of the MAFs values as positive
        if(sum(test$x[,j]>=0)/length(test$x[,j])<0.5){
            test$x[,j] <- test$x[,j] * -1
            test$rotation[,j] <- test$rotation[,j] * -1
        }
                
        # the following realisations will be order so as to have 50% of the MAFs values 
        # with the same sign than the reference 
        if(sum((test$x[,j]>=0)==(resmaf$x[,j]>=0),na.rm=T)/length(test$x[,j])<0.5){
            resmaf$x[,j] <- resmaf$x[,j] * -1
            resmaf$rotation[,j] <- resmaf$rotation[,j] * -1
        }
    }
                    
    # save results in outputs
    t.l <- seq(t[1],t[length(t)],1)                        
    for(j in 1:lng){
        res.noise[[j]][[1]] <- rbind(res.noise[[j]][[1]],as.numeric(resmaf$x[,j][match(t.l,t)]))
        res.noise[[j]][[2]] <- rbind(res.noise[[j]][[2]],as.numeric(resmaf$rotation[,j]))
    }
}
    
# Number of MAFs retained
nbmafs <- 2
        
# Compute and save
# - median loadings of the nreal previous realisations   
# - median MAF = median loadings * centred and normed indices     
for(i in 1:lng){
    med.rot <- apply(res.noise[[i]][[2]],2,median)
    resmaf$rotation[,i] <- med.rot
    rot <- matrix(rep(med.rot,nij[1]),nij[1],byrow=T)
    x <- apply(rot * scaled.zInd,1,sum)
    resmaf$rotation[,i] <- resmaf$rotation[,i]/sd(x)
    resmaf$x[,i] <- x/sd(x)
}

# Loop on the retained MAFs
# Plots produced for each MAF: 
# - its time series
# - its variogram
# - the loadings of each variable in the MAF (the highest is highlighted)

maf.g1 <- numeric(0)
maf.vg <- vector("list",nbmafs)

for(i in 1:nbmafs){
    x11(width=9,heigh=3)
    par(mfrow = c(1,3))
    par("mar"=c(2.5,2.5,2.5,1))
    maf.x<-as.numeric(resmaf$x[,i][match(t.l,t)])
    barplot(aperm(matrix(maf.x)),names.arg=matrix(t.l),beside=F,space=0,col=8,
        cex.names=.7,main=paste("MAF ",i," (",name,")",sep=""),
        ylim=c(-3.5,3.5))

    maf.vg[[i]]<-vg1d.reg(maf.x)
    maf.g1 <- c(maf.g1,maf.vg[[i]][1,2])
    plot(maf.vg[[i]],type="o",main="",ylim=c(0,3))
    abline(h=1,lty=2)
    title("Variogram")

    maf.rot<-resmaf$rotation[,i]
    plot(0,0,ylim=c(-max(ceiling(abs(maf.rot))),max(ceiling(abs(maf.rot)))),
        xlim=c(0,jInd),type="n",xlab="",ylab="",xaxt="n",main="Loadings")
    segments(x0=seq(0.5,jInd-.5,1),y0=rep(0,jInd),x1=seq(0.5,jInd-.5,1),y1=maf.rot)
    abline(h=0)
    Col<-rep(1,jInd)
    Col[seq(1,jInd)[abs(maf.rot)==max(abs(maf.rot))]]<-3
    text(seq(0.5,jInd-.5,1),maf.rot, names(z[,sort(match(Ind,names(z)))]),cex=.75,col=Col)

    dev.print(png, file=paste("MAF.",name,".fig",i,".png",sep=""), width=1200, height=400)
    dev.copy2eps(file=paste("MAF.",name,".fig",i,".eps",sep=""))
}
res.maf.vario.at.lag.1 <- data.frame(MAFs=paste("MAF",1:nbmafs,sep=""),Vario.at.lag.1=maf.g1)
res.maf.rotation <- resmaf$rotation[,1:nbmafs]

# Continuity of Indices
# with the first 2 MAFs
nc <- 2 
if(nc>nbmafs) {cat("nc must be < nbmafs","\n")}
conti<-apply(sweep(res.maf.rotation[,1:nc]^2,2,(1-res.ts.vario.at.lag.1[1:nc,2]),"*"),1,sum)
conti[order(conti,decreasing=T)]
res.continuity <- data.frame(continuity=conti[order(conti,decreasing=T)])

# Plots
x11(); plot(1:length(conti),conti[order(conti,decreasing=T)],type="b",
       xlab='rank of index',ylab='continuity',pch="+", 
       ylim=c(0,max(conti[order(conti,decreasing=T)])+.1),
       xlim=c(1,length(conti)+3))
text(1:length(conti),conti[order(conti,decreasing=T)],
     labels=labels(conti[order(conti,decreasing=T)]),srt=45,pos=4)

dev.print(png, file=paste("MAF.",name,".fig",nbmafs+1,".png",sep=""), width=800, height=800)
dev.copy2eps(file=paste("MAF.",name,".fig",nbmafs+1,".eps",sep=""))
          
# plot time series of selected indic
j<-match(names(conti[order(conti,decreasing=T)]),dimnames(z)[[2]])
for (i in 1:jInd){
  x11();plot(t,z[,j[i]],type="o",xlab="",ylab=dimnames(z)[[2]][j[i]]) 
  dev.print(png, file=paste("MAF.",name,".fig",nbmafs+2+i,".png",sep=""), width=800, height=800)
  dev.copy2eps(file=paste("MAF.",name,".fig",nbmafs+2+i,".eps",sep=""))
}

# Summary results
res.ts.vario.at.lag.1
res.maf.vario.at.lag.1
res.maf.rotation
res.continuity

# Clean workspace
rm(Col,conti,fo,g,g1,i,Ind,j,jInd,jln,lng,maf,maf.g1,maf.rot,
maf.vg,maf.x,med.rot,name,nbmafs,nc,new.zInd,ni,nij,nj,noise,nreal,
nz,res,res.noise,resmaf,rot,scaled.zInd,seed,t,t.l,test,tmax,va,x,z,zInd)

# Delete figures
graphics.off()


